import java.awt.Dimension;
import javax.swing.BorderFactory;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Observable;
import java.util.Observer;


public class ChatGUI extends Observable implements IChatGUI, Observer {

	private JFrame mainFrame;
	private JPanel panel;
	private JTextField myArea;
	private JTextArea chatArea;
	private JButton button;

	private static final String APP_NAME = "Sample MultiSocket Chat Application";

	ChatGUI() {
		mainFrame = new JFrame();
	
		mainFrame.setTitle(APP_NAME); /* set the title of the window */
		panel = new JPanel(); /* create a panel container where we put all everything */
		/* set the panel layout to BoxLayout with vertical orientation (all elements 
		   will be placed from tob to bottom in this container */
		panel.setLayout( new BoxLayout( panel, BoxLayout.Y_AXIS) );

		/* at the top of our panel we add a rigid space of 5 pixels. When resizing 
		 the chat window, this will remain fixed to 5 pixels and only the flexible
		 objects will resize */
		panel.add( Box.createRigidArea(new Dimension(0, 5) ));

		/* create a chat area where the text of the chat will be outputted */ 
		chatArea = new JTextArea("");
		/* select a preferred size for this window, 300px width and 400px height*/
		chatArea.setPreferredSize(new Dimension(300, 400));
		chatArea.setLineWrap(true); /* if line is too long it will be wrapped */
		/* we wrap at word style, so full word is moved on second line */
		chatArea.setWrapStyleWord(true); 
		/* create a border of 8 pixels inside the text area for nice looking */
		chatArea.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
		chatArea.setEditable(false); /* this text area is not editable usign keyboard */

		/* by default, the text area does not have a scrool bar, so we create one 
		and add the textArea to it */
		JScrollPane pane = new JScrollPane();
		pane.getViewport().add(chatArea);
		/* we add the pane containing the textArea with scroolbar to the panel */
		panel.add(pane); 
		/* again some vertical fixed space */
		panel.add( Box.createRigidArea(new Dimension(0, 5) ));

		/* we create a second pannel on the bottom where we place the input text 
		area and the send button */
		JPanel bottom = new JPanel();
		/* the layout of this panel is BoxLayout with horizontal arangement */
		bottom.setLayout( new BoxLayout( bottom, BoxLayout.X_AXIS));

		/* create a textField for inputing the text */
		myArea = new JTextField("");
		/* select a prefered size for this field */
		myArea.setPreferredSize(new Dimension(300, 30));
		/* create and border of 8 pixels around this text field */
		myArea.setBorder(BorderFactory.createEmptyBorder(8, 8, 8, 8));
		
		/* add action listener on this field. When return key is pressed this
		listener is called */
		myArea.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				/* we are trying to send a message */
				setChanged();
				notifyObservers( getMessage() );
			}
		});

		/* we add this  textField to the bottom panel */
		bottom.add(myArea);

		/* we create a horizontal rigid 5 pixel space between this text field
		 and the send button */
		bottom.add( Box.createRigidArea(new Dimension(5, 0) ));

		/* now we create a send button */
		button = new JButton("Send");
		/* we add a listener on it, that is, when it is preessed it will enter
		this routine */
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				setChanged();
				notifyObservers( getMessage() );
			}
		});

		/* we add this button to the bottom panel */
		bottom.add(button);
		/* we add the bottom pannl to the bottom of the main panel */
		panel.add(bottom);
		/* we add some fixed 5 px space at the bottom */
		panel.add( Box.createRigidArea(new Dimension(0, 5) ));
		
		/*as this object is an extension of a JFrame, we simply add this pannel to it */
		mainFrame.add(panel);  
		mainFrame.pack();

		/* we associate to pressing the X buttong the action exit */
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		/* set the location whatever */
		mainFrame.setLocationRelativeTo(null);
		/* we set it visible */
		mainFrame.setVisible(true);
	}

	/**
	 * when sendInProgress returns TRUE, this method must return the text we
	 * want to send to the other chat members;
	 * @return chat text
	 */ 
	public String getMessage(){
		/* we fetch the text from the text area */
		String temp = myArea.getText();
		/* we reset the text area */
		myArea.setText(""); 
		return temp;
	}
	
	/**
	 * when a message is received it is passed to this method for further 
	 * processing 
	 * @param message the message received from other members
	 */
	public void setMessage( String message ){
		/* we append a new line containing the message to the chatArea */
		chatArea.append("\n" + message);
	}

	/** 
	 * this method is called when an receive event happens on the chatClient object.
	 * this method simply appends the chat line to the textArea 
	 */	
	public void update (Observable obj, Object arg)
    {
        if (arg instanceof String)
        {
            String resp = (String) arg;
            setMessage(resp);
        }
    }

}

