import net.jxta.peergroup.PeerGroup;
import net.jxta.platform.NetworkManager;
import net.jxta.platform.NetworkConfigurator;
import net.jxta.socket.JxtaMulticastSocket;
import net.jxta.document.AdvertisementFactory;
import net.jxta.id.IDFactory;
import net.jxta.pipe.PipeID;
import net.jxta.pipe.PipeService;
import net.jxta.protocol.PipeAdvertisement;

import net.jxta.document.Advertisement;
import java.util.Enumeration;
import net.jxta.discovery.DiscoveryService;

import net.jxta.pipe.PipeService;

import java.io.File;
import java.io.IOException;
import java.net.DatagramPacket;
import java.util.Date;
import java.lang.*;
import java.net.URI;
import java.net.URISyntaxException;

import java.util.Scanner;

import java.util.Observable;
import java.util.Observer;  /* this is Event Handler */

/**
 * Simple example to illustrate the use of JxtaMulticastSocket
 */
public class Client extends Observable implements Runnable, Observer{

	private NetworkManager manager = null;
	private NetworkConfigurator theNetworkConfigurator = null;
	private PeerGroup thePeerGroup = null;
	
	private JxtaMulticastSocket mcastSocket = null;
	private PeerGroup netPeerGroup;
	private DiscoveryService discovery;
	private PipeService pipeService;	

	public final static String SOCKETIDSTR = "urn:jxta:uuid-59616261646162614E5047205032503393B5C2F6CA7A41FDB0F890173088E79404";
	public boolean texec = false;

	private String myID;

	/**
	* Constructor for the Client. Fetches the netPeerGroup and the discovery
	* service associated with it. Initializes the pipe service
	* @param[in] clientName The local client name, that is your ID
	*/
	public Client(String clientName) {
		
		myID = clientName;
		/* initialize the network manager */
		try{
			manager = new NetworkManager( NetworkManager.ConfigMode.ADHOC, "Chat", new File( new File(".cache"), clientName ).toURI() );  
			manager.setConfigPersistent(false);
			theNetworkConfigurator = manager.getConfigurator();
			
			if (theNetworkConfigurator.exists()){
				System.out.println(">INFO: Configuration found ...");
				File networkConfig = new File ( theNetworkConfigurator.getHome(), "PlatformConfig" );
				try{
					theNetworkConfigurator.load( networkConfig.toURI());
				}catch(Exception e){
					e.printStackTrace();
					System.exit(-1);
				}
			}else{
				System.out.println(">INFO: Configuration not found. Creating new one !");
				theNetworkConfigurator.setName( clientName);
				theNetworkConfigurator.setPrincipal ( clientName );
				theNetworkConfigurator.setPassword ("12345678");
				theNetworkConfigurator.save();
			}
			
			System.out.println(">INFO: Starting JXTA");
			thePeerGroup = manager.startNetwork();
			System.out.println(">INFO: JXTA Started");
			
			discovery = thePeerGroup.getDiscoveryService();
			pipeService = manager.getNetPeerGroup().getPipeService();
		}catch(Exception e){
			e.printStackTrace();
			System.exit(-1);
		}
	
        try {
            mcastSocket = new JxtaMulticastSocket(thePeerGroup, getSocketAdvertisement());
        } catch (IOException e) {
            e.printStackTrace();
            System.exit(-1);
        }
	}
	
	/**
	* initization of the local advertisement cache. All pipe advertisements are flushed 
	* @param verb print messy details on the scrren 
	*/
	public void initLocalCache(boolean verb){
		Advertisement adv;
		System.out.println(">INFO: Printing existing local advertisements");
		try{
			Enumeration en = discovery.getLocalAdvertisements(DiscoveryService.ADV, null, null);
			if (en != null) {
				while (en.hasMoreElements()) {
					adv = (Advertisement) en.nextElement();
					if ( adv instanceof PipeAdvertisement ){
						if (verb) 
							System.out.println(">INFO: --------- Pipe Advertisement ---------");
					}
					if (verb){
						System.out.println(">INFO: ------------------------------------");					
						System.out.println(adv.toString());
					}
					if ( adv instanceof PipeAdvertisement ){
						if (verb)
							System.out.println(">INFO: .... flushing pipe advertisement ............"	);
						discovery.flushAdvertisement(adv);
					}
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			System.exit(-1);
		}
	}

    /**
     * chat function.
     *
     * @param myID my ID :)
     */
	public void chat( String theChatLine){
		try {
			String newChatLine;
			newChatLine = "[ " + myID + " ]: " + theChatLine;
			
	        DatagramPacket packet = new DatagramPacket(newChatLine.getBytes(), newChatLine.length());
	        mcastSocket.send(packet);
	    } catch (IOException e) {
	        e.printStackTrace();
	    }
	}

	/**
	 * This thread will print messages on the screen
	 */
	public void run() {
		byte[] res = new byte[16384];
		DatagramPacket rpacket = null;

		try{
			while(true){
				// response
				rpacket = new DatagramPacket(res, res.length);
				mcastSocket.receive(rpacket);
				String sw = new String(rpacket.getData(), 0, rpacket.getLength());
				System.out.println("Received data from :" + rpacket.getAddress());
				System.out.println(sw);
				setChanged();
				notifyObservers(sw);
			}
		}catch(Exception e){
		e.printStackTrace();
		System.exit(-1);
		}
	}


    /**
     * main
     *
     * @param args command line args
     */
    public static void main(String args[]) {

		if (args.length != 1){
			System.out.println(">ERROR: Argument is your ID");
			System.exit(-1);
		}

		Client chatClient = new Client(args[0]);
		chatClient.initLocalCache(false);
		new Thread(chatClient).start();
		chatClient.chat(args[0]);
	}
    
	/**
	* Creates a Socket Pipe Advertisement with the pre-defined pipe ID
	*
	* @return a socket PipeAdvertisement
	*/
    public static PipeAdvertisement getSocketAdvertisement() {
        PipeID socketID = null;

        try {
            socketID = (PipeID) IDFactory.fromURI(new URI(SOCKETIDSTR));
        } catch (URISyntaxException use) {
            use.printStackTrace();
        }
        PipeAdvertisement advertisement = (PipeAdvertisement) AdvertisementFactory.newAdvertisement(PipeAdvertisement.getAdvertisementType());

        advertisement.setPipeID(socketID);
        // set to type to propagate
        advertisement.setType(PipeService.PropagateType);
        advertisement.setName("Multicast chat socket demo");
        return advertisement;
    }
    
	/**
	* Stops the platform
	*/
	public void stop() {
		// Stop JXTA
		manager.stopNetwork();
	}
    
	/** when an event is detected on the interface (send button is pressed) this
	 observer is notified, and it calls the chat() method for sending one message */
	public void update (Observable obj, Object arg)
    {
        if (arg instanceof String)
        {
            String resp = (String) arg;
            chat(resp);
        }
    }
}

